import { GraphOptions } from '@/helpers/interfaces/GraphOptions';
import * as echarts from 'echarts';

type EChartsOption = echarts.EChartsOption;

class Graph {
  private graphDefault = {
    color: [],
    tooltip: '{b0} - {d0}%',
  };

  private getDistinctColors(numColors: number): string[] {
    const tonalities = ['#FF0000', '#0000FF', '#00FF00', '#FF00FF', '#00FFFF', '#FFFF00', '#FFA500'];
    const randomColors: string[] = [];

    for (let i = 0; i < numColors; i += 1) {
      // Escolhe uma tonalidade da sequência com base no índice
      const tonality = tonalities[i % tonalities.length];

      // Gera valores aleatórios adicionais para ajustar a tonalidade
      const r = Math.floor(Math.random() * 51) + parseInt(tonality.substr(1, 2), 16);
      const g = Math.floor(Math.random() * 51) + parseInt(tonality.substr(3, 2), 16);
      const b = Math.floor(Math.random() * 51) + parseInt(tonality.substr(5, 2), 16);

      // Limita os valores para o intervalo de 0 a 255
      const rHex = Math.min(r, 255).toString(16).padStart(2, '0');
      const gHex = Math.min(g, 255).toString(16).padStart(2, '0');
      const bHex = Math.min(b, 255).toString(16).padStart(2, '0');

      // Concatena os valores para formar a cor final em formato hexadecimal
      const colorHex = `#${rHex}${gHex}${bHex}`;

      randomColors.push(colorHex);
    }

    return randomColors;
  }

  public createGraphPie(params: GraphOptions) {
    const options = { ...this.graphDefault, ...params };

    const missingColor = options.datasource.length - options.color.length;
    const color = [...options.color];

    if (missingColor > 0) {
      const newColors = this.getDistinctColors(missingColor);
      color.push(...newColors);
    }

    const source = [[`${options.name}`, 'Value'], ...options.datasource];
    const chartDom = document.getElementById(`${options.id}`)!;
    const myChart = echarts.init(chartDom);
    const graphOptions: EChartsOption = {
      dataset: [
        {
          source,
        },
        {
          transform: {
            type: 'filter',
            config: {
              dimension: 'Value',
              '>': '0',
              parser: 'number',
            },
          },
        },
      ],
      title: {
        text: `${options.title}`,
        left: 'center',
      },
      tooltip: {
        trigger: 'item',
        formatter: `${options.tooltip}`,
      },
      legend: {
        orient: 'vertical',
        left: 'left',
      },
      series: [
        {
          name: `${options.title}`,
          type: 'pie',
          datasetIndex: 1,
        },
      ],
      color,
    };

    myChart.setOption(graphOptions);
  }

  public createGraphBar(params: GraphOptions) {
    const options = { ...this.graphDefault, ...params };
    const missingColor = options.datasource[0].length - options.color.length;
    const color = [...options.color];

    if (missingColor > 0) {
      const newColors = this.getDistinctColors(missingColor);
      color.push(...newColors);
    }
    const chartDom = document.getElementById(params.id)!;
    const myChart = echarts.init(chartDom);
    const graphOptions = {
      legend: {
        left: 'right',
      },
      tooltip: {
        formatter: options.tooltip,
      },
      title: {
        text: params.title,
      },
      dataset: {
        source: params.datasource,
      },
      xAxis: { type: 'category' },
      yAxis: params.yAxis,
      series: [
        {
          name: 'Pedidos',
          type: 'bar',
        },
        {
          name: 'Valor',
          type: 'bar',
          yAxisIndex: 1,
        },
      ],
      color,
    };

    myChart.setOption(graphOptions);
  }

  public createGraphLine(params: GraphOptions) {
    const options = { ...this.graphDefault, ...params };
    const missingColor = options.datasource.length - options.color.length;
    const color = [...options.color];

    if (missingColor > 0) {
      const newColors = this.getDistinctColors(missingColor);
      color.push(...newColors);
    }
    const chartDom = document.getElementById(params.id)!;
    const myChart = echarts.init(chartDom);
    const graphOptions = {
      title: {
        text: options.title,
      },
      legend: {
        data: options.legendData,
        left: 'right',
      },
      grid: {
        left: '3%',
        right: '4%',
        bottom: '3%',
        containLabel: true,
      },
      xAxis: {
        type: 'category',
        boundaryGap: false,
        data: options.data,
      },
      yAxis: [
        {
          type: 'value',
        },
      ],
      tooltip: {
        trigger: 'axis',
        formatter: options.tooltip,
      },
      series: options.datasource,
      color,
    };

    myChart.setOption(graphOptions);
  }

  public createGraphPieNest(params: GraphOptions) {
    const options = { ...this.graphDefault, ...params };
    const missingColor = (options.datasource[0].length + options.datasource[1].length) - options.color.length;
    const color = [...options.color];

    if (missingColor > 0) {
      const newColors = this.getDistinctColors(missingColor);
      color.push(...newColors);
    }

    const chartDom = document.getElementById(params.id)!;
    const myChart = echarts.init(chartDom);
    const graphOptions = {
      dataset: [
        {
          source: [...options.datasource[0]],
        },
        {
          source: [...options.datasource[1]],
        },
      ],
      tooltip: {
        trigger: 'item',
        formatter: options.tooltip,
      },
      title: {
        text: options.title,
      },
      series: [
        {
          name: 'Modalidade',
          type: 'pie',
          radius: [0, '50%'],
          label: {
            fontSize: 14,
          },
          datasetIndex: 0,
        },
        {
          name: 'Origem',
          type: 'pie',
          radius: ['60%', '90%'],
          labelLine: {
            length: 30,
          },
          datasetIndex: 1,
        },
      ],
      legend: {
        orient: 'vertical',
        left: 'right',
      },
      color,
    };

    myChart.setOption(graphOptions);
  }
}

export default new Graph();
